#include "MotorGlueComponent.hh"

MotorGlueComponent::MotorGlueComponent(const dzn::locator& locator) :
    skel::MotorGlueComponent(locator),
    m_locator(locator),
    m_pump(locator.get<dzn::pump>()),
    m_motor(nullptr)
{
}

Result MotorGlueComponent::api_Initialize(std::string id)
try
{
    m_motor = &m_locator.get<Hal::Motor>(id);
    m_motor->HomingChanged([this]
    {
        m_pump(api.out.OnHomingChanged);
    });
    m_motor->MovingChanged([this]
    {
        m_pump(api.out.OnMovingChanged);
    });
    return Result::Ok;
}
catch(const std::exception& ex)
{
    LogException(ex);
    return Result::Fail;
}

void MotorGlueComponent::api_Terminate()
{
    if (m_motor)
    {
        m_motor->HomingChanged(nullptr);
        m_motor->MovingChanged(nullptr);
        m_motor = nullptr;
    }
}

Result MotorGlueComponent::api_Home()
try
{
    m_motor->Home();
    return Result::Ok;
}
catch(const std::exception& ex)
{
    LogException(ex);
    return Result::Fail;
}

Result MotorGlueComponent::api_IsHoming()
try
{
    return m_motor->IsHoming() ? Result::True : Result::False;
}
catch(const std::exception& ex)
{
    LogException(ex);
    return Result::Fail;
}

Result MotorGlueComponent::api_Move(double pos)
try
{
    m_motor->Move(pos);
    return Result::Ok;
}
catch(const std::exception& ex)
{
    LogException(ex);
    return Result::Fail;
}

Result MotorGlueComponent::api_IsMoving()
try
{
    return m_motor->IsMoving() ? Result::True : Result::False;
}
catch(const std::exception& ex)
{
    LogException(ex);
    return Result::Fail;
}

Result MotorGlueComponent::api_Stop()
try
{
    m_motor->Stop();
    return Result::Ok;
}
catch(const std::exception& ex)
{
    LogException(ex);
    return Result::Fail;
}

void MotorGlueComponent::LogException(const std::exception& ex)
{
    std::cerr << dzn::path(&dzn_meta) << " exception: " << ex.what() << "\n";
}
